/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.imp;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Collection;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JFileChooser;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.filechooser.FileView;
import org.netbeans.api.project.Project;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.filesystems.FileChooserBuilder;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle.Messages;

@ActionID(category = "File",
        id = "nl.cloudfarming.client.imp.ImportAction")
@ActionRegistration(iconBase = "nl/cloudfarming/client/imp/import.png",
        displayName = "#CTL_ImportAction")
@ActionReferences({
    @ActionReference(path = "Menu/File", position = 0)
})
@Messages({"CTL_ImportAction=Import", "ImportAction chooser title=Import file"})
public final class ImportAction extends AbstractAction {

    private static final boolean FILE_HIDING = true;
    private final Project context;
    private final static Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.imp.ImportAction");
    private final int STARTING_LEVEL = 0;
    private final int MAX_LEVEL = 2;
    private ExecutorService service;

    public ImportAction(Project context) {
        super(Bundle.ImportAction_chooser_title());
        this.context = context;
    }

    public void setService(ExecutorService service) {
        this.service = service;
    }

    @Override
    public void actionPerformed(ActionEvent ev) {
        FileChooserBuilder builder = new FileChooserBuilder(ImportAction.class);
        builder.setTitle(Bundle.ImportAction_chooser_title());
        builder.setFileHiding(FILE_HIDING);

        Collection<? extends ImportProcessor> importProcessors = Lookup.getDefault().lookupAll(ImportProcessor.class);

        //If only one found set the file filter as primairy
        if (importProcessors.size() == 1) {
            ImportProcessor importProcessor = importProcessors.iterator().next();
            FileFilter filter = new FileNameExtensionFilter(importProcessor.getFormatDescription(), importProcessor.getSupportedExtentions());
            builder.setFileFilter(filter);
        } else {
            //else add all the file filters
            for (ImportProcessor importProcessor : importProcessors) {
                FileFilter filter = new FileNameExtensionFilter(importProcessor.getFormatDescription(), importProcessor.getSupportedExtentions());
                builder.addFileFilter(filter);
            }
        }
        final JFileChooser chooser = builder.createFileChooser();
        chooser.setMultiSelectionEnabled(true);
        chooser.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (JFileChooser.APPROVE_SELECTION.equals(e.getActionCommand())) {
                    if (service == null || service.isShutdown() || service.isTerminated()) {
                        LOGGER.finest("Creating new import executor service");
                        service = Executors.newFixedThreadPool(2);
                    }
                    for (File file : chooser.getSelectedFiles()) {
                        LOGGER.log(Level.FINEST, "scheduling import for file {0}", file);
                        scheduleImport(STARTING_LEVEL, file);
                    }
                }
                if (service != null && !service.isShutdown() && !service.isTerminated()) {
                    service.shutdown();
                }
            }
        });
        chooser.setFileView(new FileView() {
            @Override
            public Icon getIcon(File f) {
                Icon icon = super.getIcon(f);
                FileObject fob = FileUtil.toFileObject(f);
                // will be null for UNC root shown at top of "Look In" combobox
                if (fob != null) {
                    for (ImportProcessor importProcessor : Lookup.getDefault().lookupAll(ImportProcessor.class)) {
                        if (importProcessor.isSupported(fob)) {
                            icon = importProcessor.getIcon();
                            break;
                        }
                    }
                }
                return icon;
            }
        });
        chooser.showOpenDialog(null);


    }

    private void scheduleImport(int level, File file) {
        // recurse if file is a folder
        if (file.isDirectory() && level <= MAX_LEVEL) {
            for (File child : file.listFiles()) {
                LOGGER.log(Level.FINEST, "ImportWorker: submit file {0} to the import executor service", file);
                scheduleImport(level++, child);
            }
        } else {
            service.submit(new ImportWorker(file));
        }
    }

    private class ImportWorker implements Runnable {

        private final File file;

        public ImportWorker(File file) {
            this.file = file;
        }

        @Override
        public void run() {
            LOGGER.log(Level.FINEST, "ImportWorker: starting import for selected file {0}", file.getAbsolutePath());

            // search for available processor
            for (ImportProcessor importProcessor : Lookup.getDefault().lookupAll(ImportProcessor.class)) {
                FileObject fo = FileUtil.toFileObject(file);
                if (fo != null && importProcessor.isSupported(fo)) {
                    LOGGER.log(Level.FINEST, "ImportWorker: Processor found for file {0}", importProcessor.toString());
                    try {
                        DataObject dataObject = DataObject.find(fo);
                        ImportSource source = dataObject.getLookup().lookup(ImportSource.class);
                        if (source != null) {
                            importProcessor.doImport(fo, context);
                        }else{
                            LOGGER.log(Level.FINEST, "Data object {0} for file {1} did not contain an import source", new Object[]{dataObject.toString(), fo.toString()});
                        }
                    } catch (DataObjectNotFoundException ex) {
                        Exceptions.printStackTrace(ex);
                    }
                    break;

                }
            }
        }
    }
}
